package de.kif.frontend.repository

import de.kif.common.Repository
import de.kif.common.RepositoryType
import de.kif.common.Serialization
import de.kif.common.model.User
import de.kif.frontend.MessageHandler
import de.westermann.kobserve.event.EventHandler
import kotlinx.serialization.DynamicObjectParser
import kotlinx.serialization.list

object UserRepository : Repository<User> {

    val prefix = js("prefix")

    override val onCreate = EventHandler<Long>()
    override val onUpdate = EventHandler<Long>()
    override val onDelete = EventHandler<Long>()

    private val parser = DynamicObjectParser()

    override suspend fun get(id: Long): User? {
        val json = repositoryGet("$prefix/api/user/$id") ?: return null
        return parser.parse(json, User.serializer())
    }

    override suspend fun create(model: User): Long {
        return repositoryPost("$prefix/api/users", Serialization.stringify(User.serializer(), model))
            ?: throw IllegalStateException("Cannot create model!")
    }

    override suspend fun update(model: User) {
        if (model.id == null) throw IllegalStateException("Cannot update model which was not created!")
        repositoryPost("$prefix/api/user/${model.id}", Serialization.stringify(User.serializer(), model))
    }

    override suspend fun delete(id: Long) {
        repositoryPost("$prefix/api/user/$id/delete")
    }

    override suspend fun all(): List<User> {
        val json = repositoryGet("$prefix/api/users") ?: return emptyList()
        return parser.parse(json, User.serializer().list)
    }

    val handler = object : MessageHandler(RepositoryType.USER) {

        override fun onCreate(id: Long) = onCreate.emit(id)

        override fun onUpdate(id: Long) = onUpdate.emit(id)

        override fun onDelete(id: Long) = onDelete.emit(id)
    }
}
